@@ -1,6 +1,6 @@
 /*
- *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2024 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   stunnel       Universal SSL tunnel
+ *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -35,6 +35,7 @@
  *   forward this exception.
  */
 
+#include "common.h"
 #include "prototypes.h"
 
 #if defined HAVE_PIPE2 && defined HAVE_ACCEPT4
@@ -48,19 +49,19 @@
 
 /**************************************** prototypes */
 
-NOEXPORT SOCKET setup_fd(SOCKET, int, const char *);
+NOEXPORT int setup_fd(int, int, char *);
 
 /**************************************** internal limit of file descriptors */
 
 #ifndef USE_FORK
 
-static SOCKET max_fds;
+static int max_fds;
 
 void get_limits(void) { /* set max_fds and max_clients */
     /* start with current ulimit */
 #if defined(HAVE_SYSCONF)
     errno=0;
-    max_fds=(SOCKET)sysconf(_SC_OPEN_MAX);
+    max_fds=sysconf(_SC_OPEN_MAX);
     if(errno)
         ioerror("sysconf");
     if(max_fds<0)
@@ -83,12 +84,12 @@ void get_limits(void) { /* set max_fds and max_clients */
         max_fds=FD_SETSIZE; /* start with select() limit */
 #endif /* select() on Unix */
 
-    /* stunnel needs at least 16 file descriptors */
+    /* stunnel needs at least 16 file desriptors */
     if(max_fds && max_fds<16)
         max_fds=16;
 
     if(max_fds) {
-        max_clients=(int)(max_fds>=256 ? max_fds*125/256 : (max_fds-6)/2);
+        max_clients=max_fds>=256 ? max_fds*125/256 : (max_fds-6)/2;
         s_log(LOG_DEBUG, "Clients allowed=%d", max_clients);
     } else {
         max_clients=0;
@@ -100,8 +101,8 @@ void get_limits(void) { /* set max_fds and max_clients */
 
 /**************************************** file descriptor validation */
 
-SOCKET s_socket(int domain, int type, int protocol, int nonblock, const char *msg) {
-    SOCKET fd;
+int s_socket(int domain, int type, int protocol, int nonblock, char *msg) {
+    int fd;
 
 #ifdef USE_NEW_LINUX_API
     if(nonblock)
@@ -118,9 +119,9 @@ SOCKET s_socket(int domain, int type, int protocol, int nonblock, const char *ms
     return setup_fd(fd, nonblock, msg);
 }
 
-SOCKET s_accept(SOCKET sockfd, struct sockaddr *addr, socklen_t *addrlen,
-        int nonblock, const char *msg) {
-    SOCKET fd;
+int s_accept(int sockfd, struct sockaddr *addr, socklen_t *addrlen,
+        int nonblock, char *msg) {
+    int fd;
 
 #ifdef USE_NEW_LINUX_API
     if(nonblock)
@@ -135,8 +136,8 @@ SOCKET s_accept(SOCKET sockfd, struct sockaddr *addr, socklen_t *addrlen,
 
 #ifndef USE_WIN32
 
-int s_socketpair(int domain, int type, int protocol, SOCKET sv[2],
-        int nonblock, const char *msg) {
+int s_socketpair(int domain, int type, int protocol, int sv[2],
+        int nonblock, char *msg) {
 #ifdef USE_NEW_LINUX_API
     if(nonblock)
         type|=SOCK_NONBLOCK;
@@ -157,7 +158,7 @@ int s_socketpair(int domain, int type, int protocol, SOCKET sv[2],
     return 0;
 }
 
-int s_pipe(int pipefd[2], int nonblock, const char *msg) {
+int s_pipe(int pipefd[2], int nonblock, char *msg) {
     int retval;
 
 #ifdef USE_NEW_LINUX_API
@@ -185,28 +186,28 @@ int s_pipe(int pipefd[2], int nonblock, const char *msg) {
 
 #endif /* USE_WIN32 */
 
-NOEXPORT SOCKET setup_fd(SOCKET fd, int nonblock, const char *msg) {
+NOEXPORT int setup_fd(int fd, int nonblock, char *msg) {
 #if !defined USE_NEW_LINUX_API && defined FD_CLOEXEC
     int err;
 #endif
 
-    if(fd==INVALID_SOCKET) {
+    if(fd<0) {
         sockerror(msg);
-        return INVALID_SOCKET;
+        return -1;
     }
 #ifndef USE_FORK
     if(max_fds && fd>=max_fds) {
-        s_log(LOG_ERR, "%s: FD=%ld out of range (max %d)",
-            msg, (long)fd, (int)max_fds);
+        s_log(LOG_ERR, "%s: FD=%d out of range (max %d)",
+            msg, fd, max_fds);
         closesocket(fd);
-        return INVALID_SOCKET;
+        return -1;
     }
 #endif
 
 #ifdef USE_NEW_LINUX_API
-    (void)nonblock; /* squash the unused parameter warning */
+    (void)nonblock; /* skip warning about unused parameter */
 #else /* set O_NONBLOCK and F_SETFD */
-    set_nonblock(fd, (unsigned long)nonblock);
+    set_nonblock(fd, nonblock);
 #ifdef FD_CLOEXEC
     do {
         err=fcntl(fd, F_SETFD, FD_CLOEXEC);
@@ -217,14 +218,14 @@ NOEXPORT SOCKET setup_fd(SOCKET fd, int nonblock, const char *msg) {
 #endif /* USE_NEW_LINUX_API */
 
 #ifdef DEBUG_FD_ALLOC
-    s_log(LOG_DEBUG, "%s: FD=%ld allocated (%sblocking mode)",
-        msg, (long)fd, nonblock?"non-":"");
+    s_log(LOG_DEBUG, "%s: FD=%d allocated (%sblocking mode)",
+        msg, fd, nonblock?"non-":"");
 #endif /* DEBUG_FD_ALLOC */
 
     return fd;
 }
 
-void set_nonblock(SOCKET fd, unsigned long nonblock) {
+void set_nonblock(int fd, unsigned long nonblock) {
 #if defined F_GETFL && defined F_SETFL && defined O_NONBLOCK && !defined __INNOTEK_LIBC__
     int err, flags;
 
@@ -245,7 +246,7 @@ void set_nonblock(SOCKET fd, unsigned long nonblock) {
     if(err<0)
         sockerror("fcntl SETFL"); /* non-critical */
 #else /* WIN32 or similar */
-    if(ioctlsocket(fd, (long)FIONBIO, &nonblock)<0)
+    if(ioctlsocket(fd, FIONBIO, &nonblock)<0)
         sockerror("ioctlsocket"); /* non-critical */
 #if 0
     else
