@@ -1,24 +1,24 @@
 /*
  *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2011 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
  *   Free Software Foundation; either version 2 of the License, or (at your
  *   option) any later version.
- * 
+ *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *   See the GNU General Public License for more details.
- * 
+ *
  *   You should have received a copy of the GNU General Public License along
  *   with this program; if not, see <http://www.gnu.org/licenses>.
- * 
+ *
  *   Linking stunnel statically or dynamically with other modules is making
  *   a combined work based on stunnel. Thus, the terms and conditions of
  *   the GNU General Public License cover the whole combination.
- * 
+ *
  *   In addition, as a special exception, the copyright holder of stunnel
  *   gives you permission to combine stunnel with free software programs or
  *   libraries that are released under the GNU LGPL and with code included
@@ -26,7 +26,7 @@
  *   modified versions of such code, with unchanged license). You may copy
  *   and distribute such a system following the terms of the GNU GPL for
  *   stunnel and the licenses of the other code concerned.
- * 
+ *
  *   Note that people who make modified versions of stunnel are not obligated
  *   to grant this special exception for their modified versions; it is their
  *   choice whether to do so. The GNU General Public License gives permission
@@ -87,17 +87,12 @@ unsigned long stunnel_thread_id(void) {
     return ready_head ? ready_head->id : 0;
 }
 
-static CONTEXT *new_context(void) {
+NOEXPORT CONTEXT *new_context(void) {
     static int next_id=1;
     CONTEXT *context;
 
     /* allocate and fill the CONTEXT structure */
-    context=str_alloc(sizeof(CONTEXT));
-    if(!context) {
-        s_log(LOG_ERR, "Unable to allocate CONTEXT structure");
-        return NULL;
-    }
-    str_detach(context);
+    context=str_alloc_detached(sizeof(CONTEXT));
     context->id=next_id++;
     context->fds=NULL;
     context->ready=0;
@@ -113,21 +108,22 @@ static CONTEXT *new_context(void) {
     return context;
 }
 
-void sthreads_init(void) {
+int sthreads_init(void) {
     /* create the first (listening) context and put it in the running queue */
     if(!new_context()) {
-        s_log(LOG_ERR, "Unable create the listening context");
-        die(1);
+        s_log(LOG_ERR, "Cannot create the listening context");
+        return 1;
     }
     /* no need to initialize ucontext_t structure here
        it will be initialied with swapcontext() call */
+    return 0;
 }
 
 int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
     CONTEXT *context;
 
     (void)ls; /* this parameter is only used with USE_FORK */
-    
+
     s_log(LOG_DEBUG, "Creating a new context");
     context=new_context();
     if(!context) {
@@ -151,17 +147,7 @@ int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
     context->context.uc_link=NULL; /* stunnel does not use uc_link */
 
     /* create stack */
-    context->stack=str_alloc(arg->opt->stack_size);
-    if(!context->stack) {
-        str_free(context);
-        if(arg)
-            str_free(arg);
-        if(s>=0)
-            closesocket(s);
-        s_log(LOG_ERR, "Unable to allocate stack");
-        return -1;
-    }
-    str_detach(context->stack);
+    context->stack=str_alloc_detached(arg->opt->stack_size);
 #if defined(__sgi) || ARGC==2 /* obsolete ss_sp semantics */
     context->context.uc_stack.ss_sp=context->stack+arg->opt->stack_size-8;
 #else
@@ -170,8 +156,8 @@ int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
     context->context.uc_stack.ss_size=arg->opt->stack_size;
     context->context.uc_stack.ss_flags=0;
 
-    s_log(LOG_DEBUG, "Context %ld created", context->id);
     makecontext(&context->context, (void(*)(void))cli, ARGC, arg);
+    s_log(LOG_DEBUG, "New context created");
     return 0;
 }
 
@@ -179,8 +165,8 @@ int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
 
 #ifdef USE_FORK
 
-void sthreads_init(void) {
-    /* empty */
+int sthreads_init(void) {
+    return 0;
 }
 
 unsigned long stunnel_process_id(void) {
@@ -191,7 +177,7 @@ unsigned long stunnel_thread_id(void) {
     return 0L;
 }
 
-static void null_handler(int sig) {
+NOEXPORT void null_handler(int sig) {
     (void)sig; /* skip warning about unused parameter */
     signal(SIGCHLD, null_handler);
 }
@@ -224,7 +210,7 @@ int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
 #ifdef USE_PTHREAD
 
 static pthread_mutex_t stunnel_cs[CRIT_SECTIONS];
-static pthread_mutex_t lock_cs[CRYPTO_NUM_LOCKS];
+static pthread_mutex_t *lock_cs;
 
 void enter_critical_section(SECTION_CODE i) {
     pthread_mutex_lock(stunnel_cs+i);
@@ -234,7 +220,7 @@ void leave_critical_section(SECTION_CODE i) {
     pthread_mutex_unlock(stunnel_cs+i);
 }
 
-static void locking_callback(int mode, int type, const char *file, int line) {
+NOEXPORT void locking_callback(int mode, int type, const char *file, int line) {
     (void)file; /* skip warning about unused parameter */
     (void)line; /* skip warning about unused parameter */
     if(mode&CRYPTO_LOCK)
@@ -247,21 +233,18 @@ struct CRYPTO_dynlock_value {
     pthread_mutex_t mutex;
 };
 
-static struct CRYPTO_dynlock_value *dyn_create_function(const char *file,
+NOEXPORT struct CRYPTO_dynlock_value *dyn_create_function(const char *file,
         int line) {
     struct CRYPTO_dynlock_value *value;
 
     (void)file; /* skip warning about unused parameter */
     (void)line; /* skip warning about unused parameter */
-    value=str_alloc(sizeof(struct CRYPTO_dynlock_value));
-    if(!value)
-        return NULL;
-    str_detach(value);
+    value=str_alloc_detached(sizeof(struct CRYPTO_dynlock_value));
     pthread_mutex_init(&value->mutex, NULL);
     return value;
 }
 
-static void dyn_lock_function(int mode, struct CRYPTO_dynlock_value *value,
+NOEXPORT void dyn_lock_function(int mode, struct CRYPTO_dynlock_value *value,
         const char *file, int line) {
     (void)file; /* skip warning about unused parameter */
     (void)line; /* skip warning about unused parameter */
@@ -271,7 +254,7 @@ static void dyn_lock_function(int mode, struct CRYPTO_dynlock_value *value,
         pthread_mutex_unlock(&value->mutex);
 }
 
-static void dyn_destroy_function(struct CRYPTO_dynlock_value *value,
+NOEXPORT void dyn_destroy_function(struct CRYPTO_dynlock_value *value,
         const char *file, int line) {
     (void)file; /* skip warning about unused parameter */
     (void)line; /* skip warning about unused parameter */
@@ -284,10 +267,20 @@ unsigned long stunnel_process_id(void) {
 }
 
 unsigned long stunnel_thread_id(void) {
+#if defined(SYS_gettid) && defined(__linux__)
+    return syscall(SYS_gettid);
+#else
     return (unsigned long)pthread_self();
+#endif
 }
 
-void sthreads_init(void) {
+#if OPENSSL_VERSION_NUMBER>=0x10000000L
+NOEXPORT void threadid_func(CRYPTO_THREADID *tid) {
+    CRYPTO_THREADID_set_numeric(tid, stunnel_thread_id());
+}
+#endif
+
+int sthreads_init(void) {
     int i;
 
     /* initialize stunnel critical sections */
@@ -295,15 +288,22 @@ void sthreads_init(void) {
         pthread_mutex_init(stunnel_cs+i, NULL);
 
     /* initialize OpenSSL locking callback */
-    for(i=0; i<CRYPTO_NUM_LOCKS; i++)
+    lock_cs=str_alloc_detached(CRYPTO_num_locks()*sizeof(pthread_mutex_t));
+    for(i=0; i<CRYPTO_num_locks(); i++)
         pthread_mutex_init(lock_cs+i, NULL);
+#if OPENSSL_VERSION_NUMBER>=0x10000000L
+    CRYPTO_THREADID_set_callback(threadid_func);
+#else
     CRYPTO_set_id_callback(stunnel_thread_id);
+#endif
     CRYPTO_set_locking_callback(locking_callback);
 
     /* initialize OpenSSL dynamic locks callbacks */
     CRYPTO_set_dynlock_create_callback(dyn_create_function);
     CRYPTO_set_dynlock_lock_callback(dyn_lock_function);
     CRYPTO_set_dynlock_destroy_callback(dyn_destroy_function);
+
+    return 0;
 }
 
 int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
@@ -311,7 +311,7 @@ int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
     pthread_attr_t pth_attr;
     int error;
 #if defined(HAVE_PTHREAD_SIGMASK) && !defined(__APPLE__)
-    /* Disabled on OS X due to strange problems on Mac OS X 10.5
+    /* disabled on OS X due to strange problems on Mac OS X 10.5
        it seems to restore signal mask somewhere (I couldn't find where)
        effectively blocking signals after first accepted connection */
     sigset_t new_set, old_set;
@@ -351,7 +351,7 @@ int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
 #ifdef USE_WIN32
 
 static CRITICAL_SECTION stunnel_cs[CRIT_SECTIONS];
-static CRITICAL_SECTION lock_cs[CRYPTO_NUM_LOCKS];
+static CRITICAL_SECTION *lock_cs;
 
 void enter_critical_section(SECTION_CODE i) {
     EnterCriticalSection(stunnel_cs+i);
@@ -361,7 +361,7 @@ void leave_critical_section(SECTION_CODE i) {
     LeaveCriticalSection(stunnel_cs+i);
 }
 
-static void locking_callback(int mode, int type, const char *file, int line) {
+NOEXPORT void locking_callback(int mode, int type, const char *file, int line) {
     (void)file; /* skip warning about unused parameter */
     (void)line; /* skip warning about unused parameter */
     if(mode&CRYPTO_LOCK)
@@ -374,21 +374,18 @@ struct CRYPTO_dynlock_value {
     CRITICAL_SECTION mutex;
 };
 
-static struct CRYPTO_dynlock_value *dyn_create_function(const char *file,
+NOEXPORT struct CRYPTO_dynlock_value *dyn_create_function(const char *file,
         int line) {
     struct CRYPTO_dynlock_value *value;
 
     (void)file; /* skip warning about unused parameter */
     (void)line; /* skip warning about unused parameter */
-    value=str_alloc(sizeof(struct CRYPTO_dynlock_value));
-    if(!value)
-        return NULL;
-    str_detach(value);
+    value=str_alloc_detached(sizeof(struct CRYPTO_dynlock_value));
     InitializeCriticalSection(&value->mutex);
     return value;
 }
 
-static void dyn_lock_function(int mode, struct CRYPTO_dynlock_value *value,
+NOEXPORT void dyn_lock_function(int mode, struct CRYPTO_dynlock_value *value,
         const char *file, int line) {
     (void)file; /* skip warning about unused parameter */
     (void)line; /* skip warning about unused parameter */
@@ -398,7 +395,7 @@ static void dyn_lock_function(int mode, struct CRYPTO_dynlock_value *value,
         LeaveCriticalSection(&value->mutex);
 }
 
-static void dyn_destroy_function(struct CRYPTO_dynlock_value *value,
+NOEXPORT void dyn_destroy_function(struct CRYPTO_dynlock_value *value,
         const char *file, int line) {
     (void)file; /* skip warning about unused parameter */
     (void)line; /* skip warning about unused parameter */
@@ -414,7 +411,7 @@ unsigned long stunnel_thread_id(void) {
     return GetCurrentThreadId() & 0x00ffffff;
 }
 
-void sthreads_init(void) {
+int sthreads_init(void) {
     int i;
 
     /* initialize stunnel critical sections */
@@ -422,7 +419,8 @@ void sthreads_init(void) {
         InitializeCriticalSection(stunnel_cs+i);
 
     /* initialize OpenSSL locking callback */
-    for(i=0; i<CRYPTO_NUM_LOCKS; i++)
+    lock_cs=str_alloc_detached(CRYPTO_num_locks()*sizeof(CRITICAL_SECTION));
+    for(i=0; i<CRYPTO_num_locks(); i++)
         InitializeCriticalSection(lock_cs+i);
     CRYPTO_set_locking_callback(locking_callback);
 
@@ -430,6 +428,8 @@ void sthreads_init(void) {
     CRYPTO_set_dynlock_create_callback(dyn_create_function);
     CRYPTO_set_dynlock_lock_callback(dyn_lock_function);
     CRYPTO_set_dynlock_destroy_callback(dyn_destroy_function);
+
+    return 0;
 }
 
 int create_client(int ls, int s, CLI *arg, void *(*cli)(void *)) {
@@ -459,7 +459,8 @@ void leave_critical_section(SECTION_CODE i) {
     DosExitCritSec();
 }
 
-void sthreads_init(void) {
+int sthreads_init(void) {
+    return 0;
 }
 
 unsigned long stunnel_process_id(void) {
@@ -516,12 +517,12 @@ void _endthread(void) {
 #ifdef DEBUG_STACK_SIZE
 
 #define STACK_RESERVE (STACK_SIZE/8)
-#define VERIFY_AREA ((STACK_SIZE-STACK_RESERVE)/sizeof(u32))
+#define VERIFY_AREA ((STACK_SIZE-STACK_RESERVE)/sizeof(uint32_t))
 #define TEST_VALUE 0xdeadbeef
 
 /* some heuristic to determine the usage of client stack size */
 void stack_info(int init) { /* 1-initialize, 0-display */
-    u32 table[VERIFY_AREA];
+    uint32_t table[VERIFY_AREA];
     int i, num;
     static int min_num=VERIFY_AREA;
 
@@ -549,10 +550,10 @@ void stack_info(int init) { /* 1-initialize, 0-display */
         s_log(LOG_NOTICE,
             "stack_info: size=%d, current=%d (%d%%), maximum=%d (%d%%)",
             STACK_SIZE,
-            (int)((VERIFY_AREA-num)*sizeof(u32)),
-            (int)((VERIFY_AREA-num)*sizeof(u32)*100/STACK_SIZE),
-            (int)((VERIFY_AREA-min_num)*sizeof(u32)),
-            (int)((VERIFY_AREA-min_num)*sizeof(u32)*100/STACK_SIZE));
+            (int)((VERIFY_AREA-num)*sizeof(uint32_t)),
+            (int)((VERIFY_AREA-num)*sizeof(uint32_t)*100/STACK_SIZE),
+            (int)((VERIFY_AREA-min_num)*sizeof(uint32_t)),
+            (int)((VERIFY_AREA-min_num)*sizeof(uint32_t)*100/STACK_SIZE));
     }
 }
 
